/*************************************************************************************************

//  File Name       : count_check.v 
//
//  Description     : This module implements the count checker

//  Targeted device : Family           Die          Package
//                    IGLOO2           M2GL010T     484 FBGA
//  Author          : Solutions Team
//                              

   COPYRIGHT 2013 BY MICROSEMI 
   THE INFORMATION CONTAINED IN THIS DOCUMENT IS SUBJECT TO LICENSING RESTRICTIONS 
   FROM MICROSEMI CORP.  IF YOU ARE NOT IN POSSESSION OF WRITTEN AUTHORIZATION FROM 
   MICROSEMI FOR USE OF THIS FILE, THEN THE FILE SHOULD BE IMMEDIATELY DESTROYED AND 
   NO BACK-UP OF THE FILE SHOULD BE MADE. 

 
***************************************************************************************************/

//******************************************************************************
//**************************** Module Declaration ******************************
//******************************************************************************
module count_check (
  reset_n                                                 ,
  clk                                                   ,
  sys_clk						,
  k_in,
  data_in                                               ,
  rx_val                                                ,
  clr_err_counter                                       ,
  error_out                                             ,
  lock                                                  ,
  error_count
  )                                                     ;

  input reset_n                                         ;
  input clk                                             ;
  input sys_clk                                         ;
  input [1:0] k_in;
  input [15:0] data_in                                  ;
  input rx_val                                          ;
  input clr_err_counter                                 ;
  output error_out                                      ;
  output lock                                           ;
  output [5:0] error_count                              ;

  reg shift;
  reg [7:0] c;
  reg [1:0] k;
  reg [15:0] d0;
  reg cnt_lock;

  reg [5:0]  reg_error                                  ;
  reg reg_error_out                                     ; 
  reg reg_lock                                          ;
  reg [1:0] lock_count /* synthesis syn_noprune=1 */    ;

  reg	clear_error_d1				        ;
  reg	clear_error				        ;
  reg	clear_error_s				        ;

  reg	[5:0]	tx_data;
  reg		tx_err;
  reg		tx_lock;
  reg		tx_req;
  reg	[5:0]	error_count;
  reg		error_out;
  reg		lock;
  reg		rx_ack;
  reg		tx_req_d1;
  reg		tx_req_d2;
  reg		rx_ack_d1;
  reg		rx_ack_d2;


//******************************************************************************
//*************************** Concurrent Assignment ****************************
//******************************************************************************

  always @(posedge sys_clk or negedge reset_n)
  begin
	  if (~reset_n)
		  clear_error_s <= 1'b0;
	  else
		  clear_error_s <= clr_err_counter;
  end

  always @(posedge clk or negedge reset_n)
  begin
	  if (~reset_n) begin
		  clear_error <= 1'b0;
		  clear_error_d1 <= 1'b0;
	  end
	  else
	  begin
		  clear_error_d1 <= clear_error_s;
		  clear_error <= clear_error_d1;
	  end
  end



// determine word alignment
always @(posedge clk or negedge reset_n)
begin
	if (!reset_n)
	begin
		d0 <= 16'd0;
		k <= 2'b00;
        cnt_lock <= 1'b0;
        c <= 8'd0;
        shift <= 1'b0;
	end
	else 
	begin
      case (shift)
      1'b0: // perfect alignment from CorePCS
      begin
        d0 <= data_in;
        k <= k_in;
      end
      default:
      begin
      end
      endcase   

      if (k == 2'b11)
         c <= 8'd0;
      else
      begin
         c <= c + 1;
         cnt_lock <= (d0 == {c,c});
      end
     
       
	end
end




//******************************************************************************
//************************ Logic for Checker Lock ******************************
//******************************************************************************            
  always @ (posedge clk or negedge reset_n) begin
    if (~reset_n) begin
      lock_count<=2'b00                                 ;
      reg_lock<=1'b0                                    ;
    end
    else if (rx_val==1'b1 && cnt_lock) begin

      lock_count<=lock_count+1                          ;
      if(lock_count>=2'b10) begin
        reg_lock<=1'b1                                  ;
      end
      else                                              ;
    end
      
    else if (rx_val==1'b0 || reg_error==6'b111111) begin
      lock_count<=2'b00                                 ;
      reg_lock<=1'b0                                    ;
    end
      
    else                                                ;
  end
  
//******************************************************************************
//*********************** Logic for Error Counter ******************************
//******************************************************************************     
  always @ (posedge clk or negedge reset_n) begin
    if (~reset_n) begin
      reg_error<=6'b000000                              ;
      reg_error_out<=1'b1                               ;
    end
    else if(rx_val==1'b1 && clear_error==1'b0  && ~cnt_lock) begin

      reg_error<=reg_error+1                            ;
      if (reg_error>=6'b10)
        reg_error_out<=1'b1                             ;
      else 
        reg_error_out<= reg_error_out                   ; 
      end
    else if (clear_error==1'b1) begin
      reg_error<=6'b000000                              ;
      reg_error_out<=1'b0                               ;
    end
      else                                              ;
      
  end


//******************************************************************************
//******************************* CLOCK Crossing *******************************
//******************************************************************************

always @(posedge clk or negedge reset_n)
begin
	if (~reset_n)
	begin
		tx_data <= 6'h00;
		tx_err  <= 1'b0;
		tx_lock <= 1'b0;
		tx_req <= 1'b0;
	end
	else if (~tx_req)
	begin
		tx_data <= reg_error;
		tx_err  <= reg_error_out;
		tx_lock <= reg_lock;
		tx_req <= 1'b1;
	end
	else if (rx_ack_d2)
	begin
		tx_req <= 1'b0;
	end
	else
	begin
		tx_data <= tx_data;
		tx_err  <= tx_err;
		tx_lock <= tx_lock;
	end

end

always @(posedge sys_clk or negedge reset_n)
begin
	if (~reset_n)
	begin
		error_count <= 6'h7F;
		error_out <= 1'b1;
		lock	  <= 1'b0;
		rx_ack <= 1'b0;
	end
	else if (tx_req_d2)
	begin
		error_count <= tx_data;
		error_out <= tx_err;
		lock <= tx_lock;
		rx_ack <= 1'b1;
	end
	else
	begin
		rx_ack <= 1'b0;
	end
end

always @(posedge sys_clk)
begin
	tx_req_d1 <= tx_req;
	tx_req_d2 <= tx_req_d1;
end

always @(posedge clk)
begin
	rx_ack_d1 <= rx_ack;
	rx_ack_d2 <= rx_ack_d1;
	
end


//******************************************************************************
//******************************* End of Module ********************************
//******************************************************************************
endmodule